#!/bin/bash
set -e

#
# Configure the databases for the locally running demo (creates the schema and
# tables needed).
#
# Waits for each database to become available by running a simple "SELECT READY" command.
# In the case of Oracle, as sqlplus is available before the database has started,
# it checks for the instance to be OPEN as well.
#
# Once the database is available, the DB-specific initialization script is copied
# to the containers and is run.
#

wait_for_ready() {
  attempts=1
  RESULT=$(eval "$cmd" | grep READY || true)
  while [[ -z ${RESULT} ]]; do
      ((attempts++))
      if [[ attempts -gt 30 ]]; then
          echo "Max wait time exceeded waiting for database to start"
          exit 1
      fi

    echo 'Sleeping for 5 seconds ... '
    sleep 5s
    RESULT=$(eval "$cmd" | grep READY || true)
  done
}

DIR=$(dirname "$0")

# North America
echo "Initializing North America - PostgreSQL"
cmd="docker exec north-america-postgres-db psql -U postgres -c \"SELECT 'READY'\""
wait_for_ready
docker cp "$DIR"/postgresql.sql north-america-postgres-db:/biz_init.sql
docker exec north-america-postgres-db psql -U postgres -f /biz_init.sql

# Asia
echo "Initializing Asia - PostgreSQL"
cmd="docker exec asia-postgres-db psql -U postgres -c \"SELECT 'READY'\""
wait_for_ready
docker cp "$DIR"/postgresql.sql asia-postgres-db:/biz_init.sql
docker exec asia-postgres-db psql -U postgres -f /biz_init.sql

# Europe West
echo "Initializing Europe West - SQL Server"
cmd="docker exec europe-west-sql-server-db /opt/mssql-tools/bin/sqlcmd -U sa -P Str0ngPwd! -Q \"SELECT 'READY'\""
wait_for_ready
docker cp "$DIR"/sqlserver.sql europe-west-sql-server-db:/biz-init.sql
docker exec europe-west-sql-server-db /opt/mssql-tools/bin/sqlcmd -U sa -P Str0ngPwd! -i biz-init.sql

# Europe East
echo "Initializing Europe East - MySQL"
cmd="docker exec -i europe-east-mysql-db mysql --user=root --password=password biz <<EOF
SELECT 'READY';
EOF"
wait_for_ready
docker cp "$DIR"/mysql.sql europe-east-mysql-db:/biz-init.sql
docker exec europe-east-mysql-db sh -c "mysql --user=root --password=password biz < /biz-init.sql"