import json
import os
import sys

from functions import RequestError
from functions import get_token
from functions import patch_domain
from functions import post_domain

assert sys.version_info >= (3, 7)

#
# Script to create the domains, if they dont' exist.
#

script_path = os.path.dirname(os.path.realpath(__file__))
domains_path = script_path + "/../domains/"


def create_domain(domain_file, global_file):
    f = open(domain_file, "r")
    gf = open(global_file, "r")

    try:
        json_in = json.load(f)

        print("\n*** DOMAIN INFORMATION ***")
        print("Name:               " + json_in["name"])
        print("Zone UUID:          " + json_in["zoneUuid"])
        print("Model Schema:       " + json.dumps(json_in["modelSchema"]))
        print("Dr Key Props:       " + json.dumps(json_in["drKeyProperties"]) + "\n")
        if "matchingAlgorithm" in json_in:
            print("Matching Algorithm: " + json.dumps(json_in["matchingAlgorithm"]) + "\n")

        # Get the token

        json_global = json.load(gf)
        values = json_global.get("values")
        token_href = next((x.get("value") for x in values if x.get("key") == "token_href"), None)
        client_id = next((x.get("value") for x in values if x.get("key") == "client_id"), None)
        client_secret = next((x.get("value") for x in values if x.get("key") == "client_secret"), None)
        username = next((x.get("value") for x in values if x.get("key") == "dgs_username"), None)
        password = next((x.get("value") for x in values if x.get("key") == "dgs_password"), None)
        dgs_token = get_token(token_href, client_id, client_secret, username, password)
        if dgs_token is None:
            print("Either the YOUnite server is down or the credentials for the following user are invalid: dgs")

        # Create the domain and version

        href = next((x.get("value") for x in values if x.get("key") == "href"), None)
        post_domain(href, dgs_token, json_in)

    finally:
        f.close()
        gf.close()


def update_domain(domain_name, domain_version, domain_file, global_file):
    f = open(domain_file, "r")
    gf = open(global_file, "r")

    try:
        json_in = json.load(f)

        print("\n*** DOMAIN PATCH INFORMATION ***")
        print("Name:               " + domain_name)
        print("Version  :          " + str(domain_version))
        print("Model Schema:       " + json.dumps(json_in["modelSchema"]))

        # Get the token
        json_global = json.load(gf)
        values = json_global.get("values")
        token_href = next((x.get("value") for x in values if x.get("key") == "token_href"), None)
        client_id = next((x.get("value") for x in values if x.get("key") == "client_id"), None)
        client_secret = next((x.get("value") for x in values if x.get("key") == "client_secret"), None)
        username = next((x.get("value") for x in values if x.get("key") == "dgs_username"), None)
        password = next((x.get("value") for x in values if x.get("key") == "dgs_password"), None)
        dgs_token = get_token(token_href, client_id, client_secret, username, password)
        if dgs_token is None:
            print("Either the YOUnite server is down or the credentials for the following user are invalid: dgs")

        href = next((x.get("value") for x in values if x.get("key") == "href"), None)
        patch_domain(href, dgs_token, domain_name, domain_version, json_in)

    finally:
        f.close()
        gf.close()


def create_domains(global_file):
    print("Creating Customer domain and version")
    create_domain(domains_path + "customer.json", global_file)

    print("Creating Department domain and version")
    create_domain(domains_path + "department.json", global_file)

    print("Creating Employee Classification domain and version")
    create_domain(domains_path + "employee_classification.json", global_file)

    print("Creating Employee Department domain and version")
    create_domain(domains_path + "employee_department.json", global_file)

    print("Creating Employee domain and version")
    create_domain(domains_path + "employee.json", global_file)

    print("Creating Manufacturer domain and version")
    create_domain(domains_path + "manufacturer.json", global_file)

    print("Creating Product domain and version")
    create_domain(domains_path + "product.json", global_file)

    print("Creating Promotion domain and version")
    create_domain(domains_path + "promotion.json", global_file)

    print("Creating Purchase Order domain and version")
    create_domain(domains_path + "purchase_order.json", global_file)

    print("Creating Sale domain and version")
    create_domain(domains_path + "sale.json", global_file)

    try:
        print("Patching Employee Department")
        update_domain("employeeDepartment", 1, domains_path + "employee_department_patch.json", global_file)
    except RequestError:
        # ignore as subsequent PATCH calls will produce a 400 bad request
        pass

    try:
        print("Patching Customer")
        update_domain("customer", 1, domains_path + "customer_patch.json", global_file)
    except RequestError:
        # ignore as subsequent PATCH calls will produce a 400 bad request
        pass

    try:
        print("Patching Employee")
        update_domain("employee", 1, domains_path + "employee_patch.json", global_file)
    except RequestError:
        # ignore as subsequent PATCH calls will produce a 400 bad request
        pass

    try:
        print("Patching Purchase Order")
        update_domain("purchaseOrder", 1, domains_path + "purchase_order_patch.json", global_file)
    except RequestError:
        # ignore as subsequent PATCH calls will produce a 400 bad request
        pass
