#!/usr/bin/python3

import getopt
import json
import os
import sys

import load
from functions import get_token
from functions import get_zone
from functions import post_outbound_acl

assert sys.version_info >= (3, 7)

def usage():
    print("")
    print("Use this to create two different DRs in two zones that will be ambiguous with a third DR.")
    print("")

try:
    opts, _ = getopt.getopt(sys.argv[1:], "h")
except getopt.GetoptError as err:
    print("ERROR: " + err.msg + "\n")
    usage()
    sys.exit(1)

script_path = os.path.dirname(os.path.realpath(__file__))

global_file = script_path + "/../local_global.json"
namespace = "default"
auth_server_url = "http://younite-api:8080"

for o, _ in opts:
    if o == "-h":
        usage()
        sys.exit(0)

try:
    gf = open(global_file, "r")
    # Get the token
    json_global = json.load(gf)
    values = json_global.get("values")

    token_href = next((x.get("value") for x in values if x.get("key") == "token_href"), None)
    client_id = next((x.get("value") for x in values if x.get("key") == "client_id"), None)
    client_secret = next((x.get("value") for x in values if x.get("key") == "client_secret"), None)
    username = next((x.get("value") for x in values if x.get("key") == "dgs_username"), None)
    password = next((x.get("value") for x in values if x.get("key") == "dgs_password"), None)

    dgs_token = get_token(token_href, client_id, client_secret, username, password)
    if dgs_token is None:
        print("Either the YOUnite server is down or the credentials for the following user are invalid: dgs")
    href = next((x.get("value") for x in values if x.get("key") == "href"), None)

    # Get Asia Zone
    asia_zone_uuid = get_zone(href, dgs_token, 'Asia Headquarters')
    print ('asia_zone_uuid=',asia_zone_uuid)

    # Get Europe  Zone
    eu_zone_uuid = get_zone(href, dgs_token, 'Europe Headquarters')
    print ('eu_zone_uuid=',eu_zone_uuid)

    # Get NA Zone
    na_zone_uuid = get_zone(href, dgs_token, 'North America Headquarters')
    print ('na_zone_uuid=',na_zone_uuid)

    print("Block data events from flowing out of zones by adding Outbound ACLs from each zone")

    # Set ACLs for Asia
    asia_acl_response = post_outbound_acl(href, dgs_token,
                               asia_zone_uuid,
                               "00000000-0000-0000-0000-000000000000",
                               "00000000-0000-0000-0000-000000000000",
                               "00000000-0000-0000-0000-000000000000",
                               "00000000-0000-0000-0000-000000000000",
                               False, True, True, True)

    # Set ACLs for EU
    eu_acl_response = post_outbound_acl(href, dgs_token,
                                        eu_zone_uuid,
                                        "00000000-0000-0000-0000-000000000000",
                                        "00000000-0000-0000-0000-000000000000",
                                        "00000000-0000-0000-0000-000000000000",
                                        "00000000-0000-0000-0000-000000000000",
                                        False, True, True, True)
    # Set ACLs for NA
    na_acl_response = post_outbound_acl(href, dgs_token,
                                      na_zone_uuid,
                                      "00000000-0000-0000-0000-000000000000",
                                      "00000000-0000-0000-0000-000000000000",
                                      "00000000-0000-0000-0000-000000000000",
                                      "00000000-0000-0000-0000-000000000000",
                                      False, True, True, True)

    # Load DR in Asia adaptor

    print("Load data...")

    load.load_data("1", script_path + "/../test-data/employee_ambiguous_script1.sql", post=False)

    # Load DR in EU adaptor
    load.load_data("2", script_path + "/../test-data/employee_ambiguous_script2.sql", post=False)

    # Load DR in NA adaptor - this will be the ambiguous with both of the above
    load.load_data("3", script_path + "/../test-data/employee_ambiguous_manual.sql", post=False)

    print()
    print("WARNING: An outbound restrict-all ACL has been added to each zone.")
    print("WARNING: You may want to remove them after managing the ambiguous error.")
    print()

except:
    sys.exit(0)
