
DELIMITER $$ ;

DROP PROCEDURE IF EXISTS doit;
$$

-- Set any missing service reps
CREATE PROCEDURE doit()
BEGIN
    DECLARE done INT DEFAULT FALSE;
    DECLARE id VARCHAR(25);

    DECLARE cust_cursor CURSOR FOR
        SELECT customer_id FROM customer
        WHERE sales_rep_id IS NULL;

    DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = TRUE;

    OPEN cust_cursor;
    read_loop: LOOP
        FETCH cust_cursor INTO id;
        IF done THEN
            LEAVE read_loop;
        END IF;

        UPDATE customer SET sales_rep_id = (SELECT employee_id FROM employee ORDER BY RAND() LIMIT 1)
        WHERE customer_id = id;
    END LOOP;

    CLOSE cust_cursor;
END;
$$

CALL doit();
$$

DROP PROCEDURE IF EXISTS doit;
$$

-- Add employee departments
CREATE PROCEDURE doit()
BEGIN
    DECLARE done INT DEFAULT FALSE;
    DECLARE d_id VARCHAR(25);
    DECLARE e_id VARCHAR(25);

    DECLARE employee_cursor CURSOR FOR
        SELECT employee_id FROM employee
        WHERE employee_id NOT IN (SELECT employee_id FROM employee_department);

    DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = TRUE;

    OPEN employee_cursor;
    read_loop: LOOP
        FETCH employee_cursor INTO e_id;
        IF done THEN
            LEAVE read_loop;
        END IF;

        SELECT department_id INTO d_id FROM department ORDER BY RAND() LIMIT 1;
        IF d_id IS NULL THEN
            LEAVE read_loop;
        END IF;

        INSERT INTO employee_department (employee_id, department_id, employee_role)
        VALUES (e_id, d_id, UUID());
    END LOOP;

    CLOSE employee_cursor;
END;
$$

CALL doit();
$$


DROP PROCEDURE IF EXISTS doit;
$$

-- Set any missing product department values
CREATE PROCEDURE doit()
BEGIN
    DECLARE done INT DEFAULT FALSE;
    DECLARE id VARCHAR(25);

    DECLARE product_cursor CURSOR FOR
        SELECT product_id FROM product
        WHERE department_id IS NULL;

    DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = TRUE;

    OPEN product_cursor;
    read_loop: LOOP
        FETCH product_cursor INTO id;
        IF done THEN
            LEAVE read_loop;
        END IF;

        UPDATE product SET department_id = (SELECT department_id FROM department ORDER BY RAND() LIMIT 1)
        WHERE product_id = id;
    END LOOP;

    CLOSE product_cursor;
END;
$$

CALL doit();
$$

DROP PROCEDURE IF EXISTS doit;
$$
